# -*- coding: utf-8 -*-
"""
    io.py
    :copyright: © 2019 by the EAB Tech team.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at
        http://www.apache.org/licenses/LICENSE-2.0
    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""


import os
import yaml
import tempfile
import importlib.resources
import contextlib
import pathlib


def reader(file_name):

    with open(file_name, 'r') as stream:
        return yaml.safe_load(stream)


def write_tmp_file(content):

    new_file, file_name = tempfile.mkstemp()

    os.write(new_file, content.encode())
    os.close(new_file)
    return file_name


def get_artifact(template_name):

    fn_template_path = get_artifact_path(template_name)
    with fn_template_path as path:
        with path.open("r") as fd:
            return fd.read()


def get_artifact_path(artifact_name) -> contextlib.AbstractContextManager[pathlib.Path]:
    """
    Reads an artifact out of the rio-tools project.
    """
    files = importlib.resources.files("juniper")
    return importlib.resources.as_file(files.joinpath(os.path.join('artifacts', artifact_name)))



# -*- coding: utf-8 -*-
"""
    manifest.py
    :copyright: © 2019 by the EAB Tech team.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at
        http://www.apache.org/licenses/LICENSE-2.0
    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""


from pathlib import Path
from itertools import chain


def validate_manifest_definition(manifest_definition):
    """
    Raise errors if problems are found in the manifest file. It checks if
    requirements and include paths exist.
    :param manifest_definition: Dictionary of the manifest
    """
    if missing('requirements', manifest_definition, optional=True):
        raise FileNotFoundError('You have missing requirements files: '
                                f'{missing("requirements", manifest_definition)}')
    if missing('include', manifest_definition):
        raise FileNotFoundError('You have empty include paths: '
                                f'{missing("include", manifest_definition)}')


def missing(key, manifest_definition, optional=False):
    """
    Return all file paths from the key of the manifest that don't exist. Optional
    parameter still does validation when defined.
    :param key: A key in the manifest_definitions functions dictionary
    :param manifest_definition: Dictionary of the manifest
    :param optional: Identifies the key as being required or not. A required key
      will return a missing value array.
    :return: Dictionary of all paths or files that don't exist
    """

    try:
        return [f for f in all_keys(manifest_definition, key)
                if not Path(f).exists()]
    except KeyError as ke:
        return [] if optional else [key]


def all_keys(manifest_definition, key):
    """
    Return all file paths from the key of the manifest.
    :param manifest_definition: Dictionary of the manifest
    :param key: A key in the manifest_definitions functions dictionary
    :return: All paths specified for that key in the manifest file
    """

    result = [function[key] for function in manifest_definition['functions'].values()]
    return result if isinstance(result[0], str) else chain.from_iterable(result)


def filter_manifest_definition(manifest_definition, name_filter):
    """
    Filters the manifest to only include functions that partially match
    the specified filter.
    :param manifest_definition: Dictionary of the manifest
    :param name_filter: A function name specified in the manifest
    :return: Filtered manifest definition
    """
    manifest_definition['functions'] = {key: value for (key, value)
                                        in manifest_definition['functions'].items()
                                        if name_filter in key.lower()}
    return manifest_definition



import os
import click
import click_log
import pathlib
import shutil
import logging

from juniper.io import reader
from juniper.constants import DEFAULT_OUT_DIR
from juniper.actions import build_artifacts
from juniper.manifest import validate_manifest_definition, filter_manifest_definition


logger = logging.getLogger(__name__)
click_log.basic_config(logger)


@click.group()
def main():
    """
    Juniper is a packaging tool with a with a single purpose in mind:
    stream and standardize the creation of a zip artifact for a set of
    AWS lambda functions.
    """
    pass


@main.command(help="""Packages a set of lambda functions defined in a given manifest file.
                      The manifest must defined these parameters:
                      1 - The name of each function to package
                      2 - A clear path to the dependencies of each lambda function
                      3 - The actual codebase to include in each zip file
                    """)
@click.option('--manifest', '-m', default='manifest.yml', help='The configuration file to use.')
@click.option('--debug', '-d', is_flag=True, help='Run the build in debug mode.')
@click.option('--skip-clean', '-s', is_flag=True, help='Does not recreate the output directory.')
@click.option('--function-name', '-f', help='Build Lambda functions matching the specified name.')
@click_log.simple_verbosity_option(logger)
def build(manifest, debug, skip_clean, function_name):

    if debug:
        logger.setLevel(logging.DEBUG)

    try:
        if manifest == 'manifest.yml' and not pathlib.Path('manifest.yml').is_file():
            manifest = 'manifest.yaml'
        manifest_definition = reader(manifest)
        validate_manifest_definition(manifest_definition)
    except FileNotFoundError as fnf:
        logger.error(str(fnf))
    else:
        old_output = manifest_definition.get('package', {}).get('output')
        if old_output:
            logger.error("The package section in the manifest is deprecated. Please use the global section")
            return

        # Set the value of output_dir in the manifest either from the custom value
        # provided or from the default.
        output_dir = manifest_definition.get('global', {}).get('output', DEFAULT_OUT_DIR)
        manifest_definition['output_dir'] = output_dir

        # Filter the manifest if specific functions should be built.
        if function_name:
            manifest_definition = filter_manifest_definition(manifest_definition, function_name)

        # Get the local env ready before building the artifacts.
        _clean(logger, skip_clean, manifest_definition)
        build_artifacts(logger, manifest_definition)


def _clean(logger, skip, manifest):
    """
    Clean the working environment before building a new set of artifacts. The clean
    operation will completely rm -f the output directory and then it will create
    a brand new directory.

    :param logger: The logger instance used by juniper.
    :param skip: A flag that indicates weather or not the clean process should
                 be executed
    :param manifest: The manifest that contains all the build parameters.
    """

    output_dir = manifest['output_dir']

    if skip:
        logger.debug('Skipping the cleaning process.')

        # The very first time the build command is executed, if the output
        # directory does not exist, we need to create it. This does not override
        # an existing directory.
        os.makedirs(output_dir, exist_ok=True)
        return

    logger.debug(f'Executing: rm -rf {output_dir} && mkdir {output_dir}')
    shutil.rmtree(output_dir, ignore_errors=True)
    os.makedirs(output_dir)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
"""
    actions.py
    :copyright: © 2019 by the EAB Tech team.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at
        http://www.apache.org/licenses/LICENSE-2.0
    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

import os
import shutil
import subprocess
from jinja2 import Template
import functools

from juniper.constants import DEFAULT_OUT_DIR, DEFAULT_DOCKER_IMAGE
from juniper.io import (get_artifact, write_tmp_file, get_artifact_path)


def build_artifacts(logger, manifest):
    """
    Creates a .zip file for each one of the serverless functions defined in the given
    manifest definitions file. Each function must include a name, and the set of directories
    to be included in the artifact. As part of the packaging, if the given function
    has a definition of a requirements file, all the dependencies in that file will be
    included in the artifact.

    :param logger: The logger instance.
    :param manifest: The definition of the functions and global parameters as defined in the input file.
    """

    compose_fn = build_compose(logger, manifest)
    logger.debug(f'docker compose -f {compose_fn} --project-directory . run sample-lambda bash')
    try:
        # Must copy the bin directory to the client's folder structure. This directory
        # will be promtly cleaned up after the artifacts are built.
        os.makedirs('./.juni/bin', exist_ok=True)
        with get_artifact_path('package.sh') as path:
            shutil.copy(path, './.juni/bin/')
        with get_artifact_path('build_layer.sh') as path:
            shutil.copy(path, './.juni/bin/')

        # Use docker as a way to pip install dependencies, and copy the business logic
        # specified in the function definitions.
        subprocess.run(["docker", "compose", "-f", compose_fn, '--project-directory', '.', 'down', '--remove-orphans'])
        subprocess.run(["docker", "compose", "-f", compose_fn, '--project-directory', '.', 'up'])
        subprocess.run(["docker", "compose", "-f", compose_fn, '--project-directory', '.', 'down'])
    finally:
        shutil.rmtree('./.juni', ignore_errors=True)


def build_compose(logger, manifest):
    """
    Builds a docker compose file with the lambda functions defined in the manifest.
    The definition of the lambda functions includes the name of the function as
    well as the set of dependencies to include in the packaging.

    :param logger: The logger instance.
    :param manifest: The yaml file that contains the info of the functions to package.
    """

    compose = _get_compose_template(manifest)
    # Returns the name of the temp file that has the docker compose definition.
    return write_tmp_file(compose)


def _get_compose_template(manifest):
    """
    Build the service entry for each one of the functions in the given context.
    Each docker compose entry will depend on the same image and it's just a static
    definition that gets built from a template. The template is in the artifacts
    folder.
    """
    artifact = get_artifact('compose-template.yml')

    def build_section(label):
        return [
            {
                'name': name,
                'image': _get_docker_image(manifest, sls_section),
                'platform': _get_platform(manifest, sls_section),
                'volumes': _get_volumes(manifest, sls_section)
            }
            for name, sls_section in manifest.get(label, {}).items()
        ]

    # Load the jinja template and build the sls functions and layers.
    return Template(artifact).render(
        functions=build_section('functions'),
        layers=build_section('layers')
    )


def _get_volumes(manifest, sls_function):
    """
    Get the docker compose volume mapping from the includes block of a serverless
    function definition. Each entry will be mapped to its own entry in the docker
    container.

    :param manifest: The entire juniper manifest.
    :param sls_function: The serverless function definition.
    """

    def get_vol(include):

        if include == './':
            return './:/var/task/common/'

        norm_include = include.rstrip('/')
        name = norm_include[norm_include.rindex('/') + 1:]
        return f'{norm_include}:/var/task/common/{name}'

    output_dir = manifest.get('output_dir', DEFAULT_OUT_DIR)
    volumes = [
        f'{output_dir}:/var/task/dist',
        './.juni/bin:/var/task/bin',
    ] + [
        get_vol(include)
        for include in sls_function.get('include', [])
    ]

    reqs_path = sls_function.get('requirements')
    if reqs_path:
        volumes.append(f'{reqs_path}:/var/task/common/requirements.txt')

    conf_path = manifest.get('global', {}).get('pipconf')
    if conf_path:
        volumes.append(f'{conf_path}:/etc/pip.conf')

    return volumes


def _get_attr(key, default, manifest, sls_function):
    """
    Get an attribute from the manifest, looking under the function first,
    then global:, and finally using a default.


    :params key: The key to retrieve
    :params default: The value to return if your key is not defined anywhere
    :params manifest: The juniper manifest file.
    :params sls_function: The serverless function definition.
    """
    function_value = sls_function.get(key)
    if function_value:
        return function_value

    global_value = manifest.get('global', {}).get(key)
    if global_value:
        return global_value

    return default


_get_docker_image = functools.partial(_get_attr, "image", DEFAULT_DOCKER_IMAGE)
_get_platform = functools.partial(_get_attr, "platform", None)



# -*- coding: utf-8 -*-
"""
    test_actions.py
    :copyright: © 2019 by the EAB Tech team.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at
        http://www.apache.org/licenses/LICENSE-2.0
    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

import yaml

from juniper import actions
from unittest.mock import MagicMock
from juniper.io import reader


logger = MagicMock()


def test_build_compose_writes_compose_definition_to_tmp_file(mocker):
    """
    The docker compose file created, is written to a tmp file. Make sure that
    the file is writen and validate that the contents of the file match the
    expected result.
    """

    tmp_filename = '/var/folders/xw/yk2rrhks1w72y0zr_7t7b851qlt8b3/T/tmp52bd77s3'
    mock_writer = mocker.patch('juniper.actions.write_tmp_file', return_value=tmp_filename)

    processor_ctx = reader('./tests/manifests/processor-test.yml')
    actual_filename = actions.build_compose(logger, processor_ctx)

    expected = read_file('./tests/expectations/processor-compose.yml')

    assert tmp_filename == actual_filename
    assert yaml.safe_load(mock_writer.call_args[0][0]) == yaml.safe_load(expected)


def test_build_artifacts_invokes_docker_commands(mocker):
    """
    Validate that the docker compose commands are executed with the valid paramters.
    Since the docker compose file was dynamically generated, we must pass the full
    path of that file to docker compose command. Also, set the context of the execution
    to the current path.
    """

    tmp_filename = '/var/folders/xw/yk2rrhks1w72y0zr_7t7b851qlt8b3/T/tmp52bd77s3'
    mock_builder = mocker.patch('juniper.actions.build_compose', return_value=tmp_filename)

    # Mocking the dependencies of this action. These three high level packages are
    # needed to invoke docker compose in the right context!
    mocker.patch('juniper.actions.os')
    mocker.patch('juniper.actions.shutil')
    mock_subprocess_run = mocker.patch('juniper.actions.subprocess.run')

    compose_cmd_calls = [
        mocker.call(["docker", "compose", "-f", tmp_filename, '--project-directory', '.', 'down', "--remove-orphans"]),
        mocker.call(["docker", "compose", "-f", tmp_filename, '--project-directory', '.', 'up']),
        mocker.call(["docker", "compose", "-f", tmp_filename, '--project-directory', '.', 'down']),
    ]

    processor_ctx = reader('./tests/manifests/processor-test.yml')
    actions.build_artifacts(logger, processor_ctx)

    mock_subprocess_run.assert_has_calls(compose_cmd_calls)
    mock_builder.assert_called_once()


def test_build_artifacts_copies_scriopts(mocker):
    """
    Since the docker compose command will be executed from within the context
    of where the lambda functions live. We need to make sure that the `package.sh`
    lives in the right context.

    Validate that a bin folder is temporarily created in the folder of the caller.
    This folder will be removed after the .zip artifacts are generated.
    """

    tmp_filename = '/var/folders/xw/yk2rrhks1w72y0zr_7t7b851qlt8b3/T/tmp52bd77s3'
    mock_builder = mocker.patch('juniper.actions.build_compose', return_value=tmp_filename)

    # Mocking the dependencies of this action. These three high level packages are
    # needed to invoke docker compose in the right context!
    mock_os = mocker.patch('juniper.actions.os')
    mock_shutil = mocker.patch('juniper.actions.shutil')
    mocker.patch('juniper.actions.subprocess.run')

    processor_ctx = reader('./tests/manifests/processor-test.yml')
    actions.build_artifacts(logger, processor_ctx)

    # Validate that this three step process is correctly executed.
    mock_os.makedirs.assert_called_with('./.juni/bin', exist_ok=True)

    assert len(mock_shutil.copy.call_args_list) == 2
    assert len(mock_shutil.copy.call_args_list[0][0]) == 2

    assert mock_shutil.copy.call_args_list[0][0][0].name == "package.sh"
    assert mock_shutil.copy.call_args_list[0][0][1] == "./.juni/bin/"

    assert mock_shutil.copy.call_args_list[1][0][0].name == "build_layer.sh"
    assert mock_shutil.copy.call_args_list[1][0][1] == "./.juni/bin/"
    mock_shutil.rmtree.assert_called_with('./.juni', ignore_errors=True)
    mock_builder.assert_called_once()


def test_build_compose_section_custom_output():
    """
    Validate that given a custom output directory, the volume mapping incldues
    the custom value instead of the default dist.
    """

    custom_output_dir = './build_not_dist'
    manifest = {
        'output_dir': custom_output_dir,
        'functions': {'test_func': {}}
    }

    result = actions._get_compose_template(manifest)
    yaml_result = yaml.safe_load(result)

    assert len([
        volume.strip()
        for volume in yaml_result['services']['test_func-lambda']['volumes']
        if custom_output_dir in volume
    ])


def test_get_volumes_fixes_trailing_slash():
    """
    If the include entry contains a trailing slash, the mapped version should NOT have it.
    """

    sls_function = {'include': ['./src/function1/']}
    manifest = {'functions': {'router': sls_function}}

    volumes = actions._get_volumes(manifest, sls_function)

    expected_mapping = './src/function1:/var/task/common/function1'
    assert expected_mapping in volumes


def test_get_volumes_with_mixed_entries():

    sls_function = {
        'include': [
            './src/common/',
            './src/benchmark',
            './src/trail',
        ]
    }
    manifest = {'functions': {'router': sls_function}}

    volumes = actions._get_volumes(manifest, sls_function)

    tmpl = './src/{name}:/var/task/common/{name}'
    assert tmpl.format(name='common') in volumes
    assert tmpl.format(name='benchmark') in volumes
    assert tmpl.format(name='trail') in volumes


def test_get_volumes_current_path():

    sls_function = {
        'include': ['./']
    }
    manifest = {'functions': {'router': sls_function}}

    volumes = actions._get_volumes(manifest, sls_function)

    assert './:/var/task/common/' in volumes


def test_get_volumes_with_files():

    sls_function = {
        'include': ['./src/lambda_function.py']
    }
    manifest = {'functions': {'router': sls_function}}

    volumes = actions._get_volumes(manifest, sls_function)

    assert './src/lambda_function.py:/var/task/common/lambda_function.py' in volumes


def test_build_compose_section_supports_layers():
    """
    Include the definition of the lambda layers in the docker compose section.
    """

    custom_output_dir = './build_not_dist'
    manifest = {
        'output_dir': custom_output_dir,
        'layers': {
            'first': {'requirements': 'requirements/first.txt'},
            'second': {'requirements': 'requirements/second.txt'},
        }
    }

    result = actions._get_compose_template(manifest)
    yaml_result = yaml.safe_load(result)

    first_layer = yaml_result['services']['first-layer']
    assert any('requirements/first.txt' in volume for volume in first_layer['volumes'])
    assert 'build_layer.sh first' in first_layer['command']

    second_layer = yaml_result['services']['second-layer']
    assert any('requirements/second.txt' in volume for volume in second_layer['volumes'])
    assert 'build_layer.sh second' in second_layer['command']


def test_build_compose_supports_layers(mocker):
    """
    Validate that given a manifest that has both functions and layers builds the
    correct docker compose template.
    """

    tmp_filename = '/var/folders/xw/yk2rrhks1w72y0zr_7t7b851qlt8b3/T/tmp52bd77s3'
    mock_writer = mocker.patch('juniper.actions.write_tmp_file', return_value=tmp_filename)

    processor_ctx = reader('./tests/manifests/layers.yml')
    actual_filename = actions.build_compose(logger, processor_ctx)

    expected = read_file('./tests/expectations/layers-compose.yml')

    assert tmp_filename == actual_filename
    assert yaml.safe_load(mock_writer.call_args[0][0]) == yaml.safe_load(expected)


def read_file(file_name):
    with open(file_name, 'r') as f:
        return f.read()



# -*- coding: utf-8 -*-
"""
    test_manifest_params.py
    :copyright: © 2019 by the EAB Tech team.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at
        http://www.apache.org/licenses/LICENSE-2.0
    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

import yaml

from juniper import actions, constants
from unittest.mock import MagicMock
from juniper.io import reader


logger = MagicMock()


def test_build_compose_sections_custom_docker_images():
    """
    Given a sample manifest file with a set of docker-image overrides,
    validate that when building the docker compose template, the result
    produces a valid template.
    """

    docker_ctx = reader('./tests/manifests/custom-docker-images.yml')
    result = actions._get_compose_template(docker_ctx)

    expected = read_file('./tests/expectations/custom-docker-images.yml')
    assert yaml.safe_load(result) == yaml.safe_load(expected)


def test_get_docker_image_default():
    """
    No customizations at all. Make sure the default docker image is used.
    """

    sls_function = {}
    manifest = {'global': {}}

    image = actions._get_docker_image(manifest, sls_function)
    assert image == constants.DEFAULT_DOCKER_IMAGE


def test_get_docker_image_global_override():
    """
    Override at the global level.
    """

    sls_function = {}
    manifest = {'global': {'image': 'python:3.6-alpine'}}

    image = actions._get_docker_image(manifest, sls_function)
    assert image == 'python:3.6-alpine'


def test_get_docker_image_funcion_level_override():

    manifest = {'global': {}}
    sls_function = {'image': 'python:3.8-alpine'}

    image = actions._get_docker_image(manifest, sls_function)
    assert image == 'python:3.8-alpine'


def test_get_docker_image_function_precedence():

    sls_function = {'image': 'python:3.8-alpine'}
    manifest = {'global': {'image': 'python:ignore_me'}}

    image = actions._get_docker_image(manifest, sls_function)
    assert image == 'python:3.8-alpine'


def test_volumes_includes_pipconf():

    sls_function = {'image': 'python:3.8-alpine'}
    manifest = {'global': {'pipconf': './path/pip.conf'}}

    volumes = actions._get_volumes(manifest, sls_function)
    assert './path/pip.conf:/etc/pip.conf' in volumes


def test_platform_default_override():
    docker_ctx = reader('./tests/manifests/custom-docker-platforms.yml')
    result = actions._get_compose_template(docker_ctx)

    expected = read_file('./tests/expectations/custom-docker-platforms.yml')
    assert yaml.safe_load(result) == yaml.safe_load(expected)


def read_file(file_name):
    with open(file_name, 'r') as f:
        return f.read()



# -*- coding: utf-8 -*-
"""
    test_manifest.py
    :copyright: © 2019 by the EAB Tech team.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at
        http://www.apache.org/licenses/LICENSE-2.0
    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

import pytest

from juniper.manifest import validate_manifest_definition, filter_manifest_definition


def test_source_in_manifest_does_not_exist_notifies_dev():

    manifest_definition = {'package': {'output': './build'},
                           'functions': {'sample': {'requirements': './requirements/dev.txt',
                                                    'include': ['./']},
                                         'another': {'requirements': './requirements/dev.txt',
                                                     'include': ['./idontexist']}}}

    with pytest.raises(FileNotFoundError) as e:
        validate_manifest_definition(manifest_definition)

    assert str(e.value) == "You have empty include paths: ['./idontexist']"


def test_requirements_in_manifest_does_not_exist_notifies_dev():
    """
    If the requirements field has a value, the files it points to, must be valid.
    """

    manifest_definition = {'package': {'output': './build'},
                           'functions': {'sample': {'requirements': './idontexist.txt',
                                                    'include': ['./']},
                                         'another': {'requirements': './requirements/dev.txt',
                                                     'include': ['./']}}}

    with pytest.raises(FileNotFoundError) as e:
        validate_manifest_definition(manifest_definition)

    assert str(e.value) == "You have missing requirements files: ['./idontexist.txt']"


def test_requirements_not_a_required_field():
    """
    A requirements file is NOT required! If it is ommited from the manifest file,
    do nothing.
    """

    manifest_definition = {'package': {'output': './build'},
                           'functions': {'sample': {'include': ['./']},
                                         'another': {'requirements': './requirements/dev.txt',
                                                     'include': ['./']}}}

    validate_manifest_definition(manifest_definition)
    assert True


def test_include_section_is_required():
    """
    Test with missing requirements section (valid) and a misspell on the includes.
    """

    manifest_definition = {'package': {'output': './build'},
                           'functions': {'sample': {'includes': ['./']},
                                         'another': {'requirements': './requirements/dev.txt',
                                                     'include': ['./']}}}
    with pytest.raises(FileNotFoundError) as e:
        validate_manifest_definition(manifest_definition)

    assert len(str(e))


def test_filter_manifest_definition():

    manifest_definition = {'package': {'output': './build'},
                           'functions': {'sample': {'includes': ['./']},
                                         'another': {'requirements': './requirements/dev.txt',
                                                     'include': ['./']}}}
    expected_manifest_definition = {'package': {'output': './build'},
                                    'functions': {'sample': {'includes': ['./']}}}

    actual_manifest_definition = filter_manifest_definition(manifest_definition, 'sample')
    assert actual_manifest_definition == expected_manifest_definition


def test_partial_filter_manifest_definition():

    manifest_definition = {'package': {'output': './build'},
                           'functions': {'sample-1': {'includes': ['./']},
                                         'sample-2': {'includes': ['./']},
                                         'another-sample': {'includes': ['./']},
                                         'another': {'requirements': './requirements/dev.txt',
                                                     'include': ['./']}}}
    expected_manifest_definition = {'package': {'output': './build'},
                                    'functions': {'sample-1': {'includes': ['./']},
                                                  'sample-2': {'includes': ['./']},
                                                  'another-sample': {'includes': ['./']}}}

    actual_manifest_definition = filter_manifest_definition(manifest_definition, 'sample')
    assert actual_manifest_definition == expected_manifest_definition



import io
import re
from collections import OrderedDict
from setuptools import setup, find_packages


with io.open('README.rst', 'rt', encoding='utf8') as f:
    readme = f.read()

# Make sure that the version of the package always matches the version of the
# tool. The version of the __init__.py is updated with the make release command.
with io.open('juniper/__init__.py', 'rt', encoding='utf8') as f:
    version = re.search(r'__version__ = \'(.*?)\'', f.read()).group(1)

with open('requirements/requirements.in', 'r') as fd:
    install_requires=fd.readlines()

setup(
    name="juniper-aidentified",
    version=version,
    author='Aidentified LLC',
    author_email='dgilman@aidentified.com',
    description="Tool to streamline the build of python lambda functions. (fork of juniper)",
    long_description=readme,
    long_description_content_type="text/x-rst",
    project_urls=OrderedDict((
        ('Code', 'https://github.com/dgilmanAIDENTIFIED/juniper'),
    )),
    license='Apache Software License',
    packages=find_packages(exclude=("tests",)),
    include_package_data=True,
    entry_points={
        'console_scripts': ['juni=juniper.cli:main'],
    },
    python_requires='>=3.9',
    test_suite="tests",
    install_requires=install_requires,
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Software Development :: Build Tools',
    ])
